﻿#pragma TextEncoding = "UTF-8"		// For details execute DisplayHelpTopic "The TextEncoding Pragma"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

////Use:
//ETMR_Tables() 									//load all Kegg .xls files
//Rename All_tables Sturm_AllTables			//Rename output
//Sturm_AllTables[][9]="ETMR2"				//Fill empty column 9 with header 'SOURCE' with actual source





//LoadTable(PathAndFile, Separator)	 loads a <separator> delimited table as a 2D text wave
//9=Tab, 10=LineFeed, 13=CarriageReturn

Function LoadTable(PathAndFile, Separator)			
	String PathAndFile,Separator
	
	Variable RefNum
	Variable Tabs, Lines, len, ii, jj
	String buffer

	Open /R RefNum as PathAndFile
	if (refNum == 0)
		return -1						// User canceled
	endif
	
	Tabs = 0
	Lines = 0

	Make/o/n=1/t/FREE w_lines


//Read lines and determine Table size	
	Do
		FReadLine refNum, buffer
		len = strlen(buffer)
		if (len == 0)
			break						// No more lines to be read
		endif
		
			
		Tabs = Max(ItemsInList(buffer,Separator), Tabs)
		
		w_lines[lines] = buffer
	
		Lines +=1
		redimension/n=(Lines+1) w_lines		//increase table size
	While (1)
	
//	Print lines, tabs					//Debug only
	
	Make/o/t/n=(lines, tabs-1) T_OutTable
	

//Assemble table	
	MultiThread T_OutTable[][]=StringFromList(q,w_lines[p],Separator)
	
	Close/A
End






Function/t SpecFile()			//Specify a file to be read

	Variable refNum
	String fnameStr=""
	Open/D/R/F="*.*" refNum	// Not a real open - just returns S_fileName
	fnameStr=S_FileName

	return fnameStr

End



Static Function/S DoOpenMultiFileDialog(fileFilters)		//Dialog to open multiple files
	String fileFilters
	Variable refNum
	String message = "Select one or more files"
	String outputPaths

	
	fileFilters += "All Files:.*;"

	Open /D /R /MULT=1 /F=fileFilters /M=message refNum
	outputPaths = S_fileName
	
	
	return outputPaths		// Will be empty if user canceled
End


//////////////////////////

Function ETMR_Tables()			//Loads and concatenates Tables
	String AllFiles
	String PathAndFile, pathStr, fileStr, saveStr, FileName, PWName
	String FileFilters = "xls Tables :.xls;"
	String Separator = Num2Char(9)								//9=Tab, 10=LineFeed, 13=CarriageReturn
	Variable NumFilesSelected, ii, len, PointPos
	Variable nLines, nTabs, maxLines = 0, maxTabs = 0
	
	//Specify files
	AllFiles = DoOpenMultiFileDialog(fileFilters)			//Get files to be loaded
	
	If(strlen(allFiles)==0)
		Print "AutoAnalysis Cancelled"
		return -1
	EndIf

	numFilesSelected = ItemsInList(AllFiles, "\r")		//number of files loaded
	
	
	
	//Start main loop 
	for(ii=0; ii<numFilesSelected; ii+=1)
	
		pathandfile = StringFromList(ii, AllFiles, "\r")
		len=strlen(pathandfile)
		PointPos = strsearch(pathandfile, ":", len, 1)
		PathStr =  pathandfile[0,PointPos-1]
		fileStr = pathandfile[Pointpos+1,len-1]
		
		len=strlen(fileStr)
		PointPos = strsearch(fileStr, ".", len, 1)
		FileName=fileStr[0,PointPos-1]
		
		Print "Reading ", FileName
		
		LoadTable(PathAndFile, Separator)
		Wave/t T_OutTable
		nLines = DimSize(T_OutTable, 0)
		nTabs = DimSize(T_OutTable, 1)
		
		Redimension /n=(-1,nTabs+1), T_OutTable
		nTabs+=1
		
		PWName=FileName[5,StrLen(FileName)-1]
		PWName=ReplaceString("_", PWName, " ")
		
		T_OutTable[][nTabs-1] = PWName
		
		maxLines += nLines
		maxTabs=max(nTabs,maxTabs)
	
		if(ii==0)
			T_OutTable[0][nTabs-1] = "PATHWAY NAME"
			duplicate/o/t T_OutTable, T_AllTables
			
		else
			DeletePoints /M=0 0,1, T_OutTable		//Delete headers
			concatenate /t/NP=0 {T_AllTables, T_OutTable}, T_Concatenated
			Duplicate/o/t T_Concatenated, T_AllTables
			Killwaves T_Concatenated
		endif
	
	
	
	endfor
	
	
	KillWaves T_OutTable
	close/a
End



Function SummaryData(Summary, Tables, outFile)
	Wave/t Summary, Tables
	String outFile
	
	Variable ii, jj, TabDim, SumDim, found
	String read, cmp
	
	Duplicate/o/free/t Tables, w_out
	
	redimension/n=(-1, 17) w_out
	
	w_out[0][10]="SIZE"
	w_out[0][11]="ES"
	w_out[0][12]="NES"
	w_out[0][13]="NOM p-val"
	w_out[0][14]="FDR q-val"
	w_out[0][15]="FWER p-val"
	w_out[0][16]="RANK AT MAX"
	
	TabDim=dimsize(w_out,0)
	SumDim=dimsize(Summary,0)
	
	For(ii=1;ii<TabDim;ii+=1)
	
	
		read=Tables[ii][8]		//Pathway Name
		
		For(jj=1;jj<SumDim;jj+=1)
		
			cmp = Summary[jj][0]
			cmp=cmp[5,StrLen(cmp)-1]
			cmp=ReplaceString("_", cmp, " ")
			
			if(StringMatch(cmp,read))
			
				w_out[ii][10]=Summary[jj][3]
				w_out[ii][11]=Summary[jj][4]
				w_out[ii][12]=Summary[jj][5]
				w_out[ii][13]=Summary[jj][6]
				w_out[ii][14]=Summary[jj][7]
				w_out[ii][15]=Summary[jj][8]
				w_out[ii][16]=Summary[jj][9]
			
			
			endif
		
		EndFor
	
	
	EndFor
	
	
	
	
	Duplicate/o w_out, $outFile
End



Function ClearEmpties(Data)
	Wave/t Data
	
	Variable xdim, ii
	string test
	
	xdim=dimsize(data,0)
	
	for(ii=xdim-1;ii>=0;ii-=1)
	
		test=data[ii][10]
		
		if(strlen(test)==0)
		
			deletepoints/m=0 ii, 1, Data
		
		endif
	
	
	endfor
	
	
	
End




Function SimplifyNames(Data, List, outFile, PWName)
	Wave/t Data, List
	String outFile, PWName
	
	Variable ii, jj, DataDim, ListDim, found = 0
	String read, cmp
	
	Duplicate/o/free/t Data, w_out
	

	
	DataDim=dimsize(w_out,0)
	ListDim=dimsize(List,0)
	
	For(ii=1;ii<DataDim;ii+=1)
	
	
		read=Data[ii][8]		//Pathway Name
		
		For(jj=0;jj<ListDim;jj+=1)
		
			cmp = List[jj]
			
			
			if(StringMatch(cmp,read))
			
				w_out[ii][8]=PWName
				found+=1
				break
			
			
			endif
		
		EndFor
	
	
	EndFor
	
	
	Printf "Replaced %g terms\r",found
	
	Duplicate/o w_out, $outFile
End



Function ClearNonEnriched(Data,col,term)
	Wave/t Data
	Variable col
	String term
	
	Variable xdim, ii, count=0
	string test
	
	xdim=dimsize(data,0)
	
	for(ii=xdim-1;ii>=0;ii-=1)
	
		test=data[ii][col]
		
		if(StringMatch(test,term))
		
			deletepoints/m=0 ii, 1, Data
			count+=1
		
		endif
	
	
	endfor
	
	
	Printf "Deleted %g lines\r", count
End


Function ClearByNumber(Data,col,comp,val)
	Wave/t Data
	Variable col, val
	String comp 
	
	
	Variable xdim, ii, count=0
	Variable test
	
	xdim=dimsize(data,0)
	
	for(ii=xdim-1;ii>=0;ii-=1)
	
		test=str2num(data[ii][col])
		
		if(test==NaN)
			continue
		endif
		
		strswitch (comp)
		
			Case "<":
				if (test<val)
					deletepoints/m=0 ii, 1, Data
				count+=1
				endif			
			Break
			
			Case "<=":
				if (test<=val)
					deletepoints/m=0 ii, 1, Data
				count+=1
				endif			
			Break
			
			Case ">":
				if (test>val)
					deletepoints/m=0 ii, 1, Data
				count+=1
				endif			
			Break
			
			Case ">=":
				if (test>=val)
					deletepoints/m=0 ii, 1, Data
				count+=1
				endif			
			Break
			
			Case "=":
			Case "==":
				if (test==val)
					deletepoints/m=0 ii, 1, Data
				count+=1
				endif			
			Break
			
			Case "!=":
			Case "=|=":
			Case "=/=":
				if (test!=val)
					deletepoints/m=0 ii, 1, Data
				count+=1
				endif			
			Break
			
			Default:
				Print "No valid argument."
				Return -1
			endswitch
	
	
	endfor
	
	
	Printf "Deleted %g lines\r", count
End




Function LabelDiseases(Data, Diseases)
	Wave/t Data, Diseases

	
	Variable ii, jj, DataDim, ListDim, found = 0
	String read, cmp
	
	String wname = NameOfWave(Data)
	
	Duplicate/o/free/t Data, w_out
	
	w_out[0][17]="DISEASE"

	
	DataDim=dimsize(w_out,0)
	ListDim=dimsize(Diseases,0)
	
	For(ii=1;ii<DataDim;ii+=1)
	
	
		read=Data[ii][8]		//Pathway Name
		
		For(jj=0;jj<ListDim;jj+=1)
		
			cmp = Diseases[jj]
			
			
			if(StringMatch(cmp,read))
			
				w_out[ii][17]="Yes"
				found+=1
				break
			else
				w_out[ii][17]="No"
			
			endif
		
		EndFor
	
	
	EndFor
	
	
	Printf "Replaced %g terms\r",found
	
	Duplicate/o w_out, $wname
End




Function FindDuplicateEntries(Data, SearchCol, outcol)
	Wave/t Data
	Variable SearchCol, outcol

	String wname, cmp, ctl, s1, s2
	Variable ii, jj, index, xDim, isBoth, nFound, nb
	
	wname = NameOfWave(Data)
	xDim = DimSize(Data,0)

	
	Duplicate/o/free/t Data, w_out
	Duplicate/o/free/t/r=[][SearchCol] Data, w_col
	
	w_out[2,xdim-1][outcol]=""			//Empty destination column
	
	
	For(ii=1;ii<xDim;ii+=1)				//Start at 1, 0 is headers
	
		Prog("FDE",ii,xDim)
	
		ctl=w_out[ii][outCol]				//Has outcol been written before?
		If(Strlen(Ctl)>0)
			Continue
		ElseIf(ii==xDim-1)	
			w_out[ii][outcol]=Data[ii][9]		//If last line is not "Both", set to source.
			break
		Endif
		
		cmp=Data[ii][SearchCol]
		s1=Data[ii][9]						//SOURCE
		index=ii+1
		isBoth=0
		nb=0
		make/o/n=1/free w_found
		w_found[0] = ii
		
		Do
			Prog("Find",index,xDim)
			FindValue /s=(index)/text=cmp w_Col
		
			if(v_value>=0)
				index=v_value
				s2=Data[v_value][9]					//SOURCE
				if(	StringMatch(s1,s2)==0)			//s1 != s2
					isBoth+=1	
					nb=0
					redimension/n=(isBoth+1) w_found
					w_found[isBoth] = v_value		
					Prog("Both",dimsize(w_found,0), xDim)	
				else
					nb+=1
					Prog("NotBoth",nb, xDim)		
				endif
				
			else			// no 2nd instance found
				w_out[ii][outcol]=Data[ii][9]		//SOURCE
				break
			endif
		
		
			index+=1
		While(index<xDim)
		
	
		if(isBoth>0)	
			For(jj=0;jj<dimsize(w_found,0);jj+=1)
				Prog("Both",dimsize(w_found,0),xDim)			
					w_out[w_found[jj]][outcol]="BOTH"
			EndFor		
		else
			For(jj=0;jj<dimsize(w_found,0);jj+=1)
				Prog("NotBoth",dimsize(w_found,0),xDim)			
					w_out[w_found[jj]][outcol]=Data[w_found[jj]][9]
			EndFor		
		endif
	

	EndFor
	

	
	
	Duplicate/o w_out, $wname
End




//////////////////


function runboth(wv)
	wave/t wv
	FindDuplicateEntries(wv, 2, 18)

	FindDuplicateEntries(wv, 8, 19)
end


